.\"
.\" Progress Meter widget man page
.\"
.\" Copyright (c) 1996 Michael A. Perzl
.\"
.\" use:
.\"    groff -Tascii -tbl -man XmProgressMeter.man
.\"
.\"    or
.\"
.\"    tbl XmProgressMeter.man | nroff -man -
.\"
.\"
.\"
.TH XmProgressMeter 3Xm "" "Version 1.0"
.\"
.\"
.\"
.\"================= NAME ===================
.\"
.SH NAME
XmProgressMeter \- The Progress Meter widget class
.\"
.\"
.\"
.\"================= SYNOPSIS ===================
.\"
.SH SYNOPSIS
.B #include <ProgressMeter.h>
.LP
Depending on your system's configuration the place of this include
file may vary.
.LP
.\"
.\"
.\"
.\"================= DESCRIPTION ===================
.\"
.SH DESCRIPTION
The ProgressMeter widget may be used to chart the completion of a
task. A ProgressMeter has an elongated rectangular region similar
to a ScrollBar. A ProgressMeter can be output only and therefore
only be set by the application.
.LP
.\"
.\"
.\"
.\"================= Classes ===================
.\"
.SS Classes
ProgressMeter inherits behaviour and resources from
\fBCore\fP and \fBPrimitive\fP classes.
.LP
The class pointer is \fBxmProgressMeterWidgetClass\fP.
.LP
The class name is \fBXmProgressMeter\fP.
.LP
.\"
.\"
.\"
.\"================= New Resources ===================
.\"
.SS New Resources
The following table defines a set of widget resources used by the
programmer to specify data. The programmer can also set the resource
values for the inherited classes to set attributes for this widget.
To reference a resource by name or by class in a .Xdefaults file,
remove the \fBXmN\fP or \fBXmC\fP prefix and use the remaining letters.
To specify one of the defined values for a resource in a .Xdefaults
file, remove the \fBXm\fP prefix and use the remaining letters (in
either lowercase or uppercase, but include any underscores between
words). The codes in the access column indicate if the given resource
can be set at creation time (C), set by using \fBXtSetValues\fP (S),
retrieved by using \fBXtGetValues\fP (G), or is not applicable (N/A).
.LP
.TS
tab(%) ;
cb s s
lb lb lb
lb lb lb
l l l .
XmProgressMeter Resource Set
_
 Name % Default % Access
   Class %   Type
_
 XmNboxMode % FALSE % CSG
   XmCBoxMode %   Boolean
_
 XmNfontList % dynamic % CSG
   XmCFontList %   XmFontList
_
 XmNmaximum % 100 % CSG
   XmCValue %   int
_
 XmNminimum % 0 % CSG
   XmCValue %   int
_
 XmNnumBoxes % 10 % CSG
   XmCValue %   int
_
 XmNorientation % XmHORIZONTAL % CSG
   XmCOrientation %   unsigned char
_
 XmNshowBoxSeparators % TRUE % CSG
   XmCShowBoxSeparators %   Boolean
_
 XmNshowPercentage % TRUE % CSG
   XmCShowPercentage %   Boolean
_
 XmNshowTime % FALSE % CSG
   XmCShowTime %   Boolean
_
 XmNshowTimeUnit % TRUE % CSG
   XmCShowTimeUnit %   Boolean
_
 XmNvalue % 50 % CSG
   XmCValue %   int
_
.TE
.\"
.\"
.\"
.TP
.B XmNboxMode
Specifies the style of the progress meter widget. If this value
is FALSE then the style will be a traditional percentage meter. If
this value is TRUE then the style will be like the progress meter
in Microsoft's new program versions (eg. WinWord 6.0).
.TP
./"
./"
./"
.B XmNfontList
Specifies the font list to use for the percentage label when
\fBXmNshowPercentage\fP is TRUE and for the time labels when
\fBXmNshowTime\fP is TRUE. If this value is NULL at initialization,
the font list is initialized by looking up the parent hierarchy
of the widget for an ancestor that is a subclass of
\fBXmBulletinBoard\fP, \fBXmVendorShell\fP or \fBXmMenuShell\fP
widget class. If such an ancestor is found, the font list is
initialized to the \fBXmNlabelFontList\fP of the ancestor widget.
If no such ancestor is found, the default is implementation dependent.
Refer to \fBXmFontList(3Xm)\fP for more information on the creation
and structure of a font list.
.TP
./"
./"
./"
.B XmNmaximum
Specifies the progress meter's maximum value.
\fBXmNmaximum\fP must be greater then \fBXmNminimum\fP.
.TP
./"
./"
./"
.B XmNminimum
Specifies the progress meter's minimum value.
\fBXmNmaximum\fP must be greater than \fBXmNminimum\fP.
.TP
./"
./"
./"
.B XmNnumBoxes
Specifies the number of boxes the progress meter should contain if
\fBXmNboxMode\fP is TRUE. This value is adapted such that the minimum
box width (horizontal layout) or height (vertical layout) is at least
5 pixels.
.TP
./"
./"
./"
.B XmNorientation
Displays the progress meter vertically or horizontally. This resource
can have values of \fBXmVERTICAL\fP and \fBXmHORIZONTAL\fP.
.TP
./"
./"
./"
.B XmNshowBoxSeparators
If this resource is TRUE and \fBXmNboxMode\fP is TRUE then the
progress meter will have shaded separators drawn in the background
color between the boxes.
.TP
./"
./"
./"
.B XmNshowPercentage
Specifies whether a percentage label for the current progress meter 
value should be displayed centered in the middle of the progress
meter. If the value is TRUE, the current progress meter value is
displayed.
.TP
./"
./"
./"
.B XmNshowTime
If this resource is TRUE then the passed time is displayed at the
lower end of the progress meter widget and the estimated remaining
time is displayed at the higher end of the progress meter widget 
with the font specified by \fBXmNfontList\fP.
.TP
./"
./"
./"
.B XmNshowTimeUnit
If \fBXmNshowTime\fP has the value FALSE, this resource does not
have any effects. Otherwise if \fBXmNshowTimeUnit\fP has the value
TRUE, the passed time and estimated remaining time labels are
displayed like this: \fB15\ sec\fP or \fB2:47\ min\fP.
If this resource is FALSE, the passed time and estimated remaining
time labels are displayed always like this: \fBmm:ss\fP where
\fBmm\fP specifies the minutes and \fBss\fP the seconds.
All the times are calculated automatically by the widget.
.TP
./"
./"
./"
.B XmNvalue
Specifies the progress meter's current position between
\fBXmNminimum\fP and \fBXmNmaximum\fP.
The value is constrained to be within these inclusive bounds.
The initial value of this resource is 50.
.LP
.\"
.\"
.\"
.\"================= Inherited Resources ===================
.\"
.SS Inherited Resources
XmProgressMeter inherits behaviour and resources from the following
superclasses. For a complete description of each resource, refer to
the man page for that superclass.
.br
\fBTHIS REST OF THIS INFORMATION HERE IS STILL MISSING !\fP
.LP
.\"
.\"
.\"
.\"================= Callback Information ===================
.\"
.SS Callback Information
A pointer to the following structure is passed to each callback:
.LP
.ta 8 16
\fBtypedef struct {
.br
	int	\fIreason\fP;
.br
	XEvent\fP	* \fIevent\fB;
.br
	int	\fIvalue\fP;
.br
} XmProgressMeterCallbackStruct;
.LP
.IP \fIreason\fP 10
Indicates the reason why the callback was invoked.
.LP
.IP \fIevent\fP 10
Points to the \fBXEvent\fP, that triggered the callback.
.LP
.IP \fIvalue\fP 10
Is the new progress meter value.
.\"
.\"
.\"
.\"================= BUGS ===================
.\"
.SH BUGS
No bugs yet found. Report bugs to mperzl@gsf.de. Please include a
detailed description and state which version of the ProgressMeter
widget you are using.
.LP
.\"
.\"
.\"
.\"================= COPYRIGHT ===================
.\"
.SH COPYRIGHT
Copyright (c) 1996 Michael A. Perzl.
.br
All rights reserved.
.LP
Permission to use, copy, modify, distribute, and sell this software
and its documentation for any purpose is hereby granted without fee,
provided that the above copyright notice appear in all copies and
that both that copyright notice and this permission notice appear in
supporting documentation, and that the name of Michael A. Perzl not
be used in advertising or publicity pertaining to distribution of
the software without specific, written prior permission.
Michael A. Perzl makes no representations about the suitability of
this software for any purpose.  It is provided "as is" without any
express or implied warranties, including, without limitation, the
implied warranties of merchantability and fitness for a particular
purpose.
.LP
Michael A. Perzl DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS.  IN NO EVENT SHALL Michael A. Perzl BE LIABLE FOR ANY
SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
.LP
.\"
.\"
.\"
.\"================= AVAILABILITY ===================
.\"
.SH AVAILABILITY
The most recent released version of the progress meter widget is always
available for anonymous ftp from
.LP
.RS 
ftp.gsf.de (146.107.1.2)
.br
/pub/mperzl
.RE
.LP
It will also be available (hopefully pretty soon) at
.LP
.RS
ftp.x.org (198.112.44.100)
.br
/contrib/widgets/motif/ProgressMeter
.RE
.LP
or from any mirror sites of ftp.x.org.
.LP
.\"
.\"
.\"
.\"================= AUTHOR ===================
.\"
.SH AUTHOR
Michael A. Perzl
.br
Institute of Radiation Protection
.br
GSF-National Research Center for Environment and Health
.br
Germany
.br
E-Mail: mperzl@gsf.de
.LP
.\"
.\"
.\"
.\"================= RELATED INFORMATION ===================
.\"
.SH RELATED INFORMATION
\fBCore(3Xm)\fP, \fBXmPrimitive(3Xm)\fP,
\fBXmCreateProgressMeter(3Xm)\fP, \fBXmProgressMeterSetValue(3Xm)\fP
and \fBXmProgressMeterGetValue(3Xm)\fP.
.\"
